'use strict';
angular.module('BadigoApp').controller('RootController', function ($rootScope, $scope, $ionicModal, $ionicLoading, $ionicPlatform, $cordovaGeolocation, firebase, $localStorage,
        $state, moment, PushService, $firebaseArray, $firebaseObject, $ionicViewSwitcher, $ionicPopup, $timeout, $cordovaBadge, $cordovaAppVersion) {

    $rootScope.rootData = {
        packageName: "com.badigo.app",
        defaultProfilePhoto: "images/noimage.png",
        defaultAlbumCoverPhoto: "images/noimage.png",
        version: "0.0.0",
        age: {
            min: 18,
            max: 55
        },
        aboutMe: {
            letterCount: 500,
            min: 0
        },
        distance: {
            min: 1,
            max: 100
        },
        gender: {
            MALE: "male",
            FEMALE: "female"
        },
        newUser: {
            USER_PROFILE_KEY: "user_profile",
            USER_PROPERTIES_KEY: "user_properties",
            USER_LOCATOIN_KEY: "location",
            USER_PROFILE_PHOTO_KEY: "profile_photo",
            USER_LIKE_KEY: "likes",
            USER_MATCHES: "matches",
            USER_CHATS: "user_chats",
            CHAT_MEMBERS: "chat_members",
            CHATS: "chats",
            CHAT_MESSAGES: "chat_messages",
            SYSTEM_LOGS: "system_logs"
        },
        defaultCoords: {
            lat: 37.774929,
            lon: -122.419416
        },
        geolocationOptions: {
            timeout: 10000,
            enableHighAccuracy: false
        },
        exploreTimeout: 3000,
        exploreMaxUser: 30,
        logType: {
            ERROR: "ERROR",
            WARN: "WARN",
            INFO: "INFO"
        },
        logTemplate: {
            type: "",
            log: "",
            created: ""
        },
        pushConfig: {
            pushURL: "https://fcm.googleapis.com/fcm/send",
            pushType: {
                NEW_MATCH: "newmatch",
                NEW_MESSAGE: "newmessage"
            }
        },
        geocode: {
            url: "http://maps.googleapis.com/maps/api/geocode/json"
        },
        staticContent: {
            rootUrl: "http://badigo.com/",
            privacyPath: "privacy.php",
            termsPath: "terms.php"
        },
        hasNewMatchOrMessages: false,
        totalNewMatch: 0,
        totalNewMessage: 0,
        matches: [],
        chats: []
    };

    var ref = firebase.database().ref();
//    firebase.database.enableLogging(true);
//    firebase.database.enableLogging(function (msg) {
//        if (msg.search(/listen on/i) !== -1) // also catches "unlisten"
//            //console.log('FB:', msg);
//    });

    $ionicPlatform.ready(function () {
        if (window.cordova && window.cordova.plugins.Keyboard) {
            FCMPlugin.onNotification(
                    function (data) {
                        if (data.wasTapped) {
                            //Notification was received on device tray and tapped by the user.
                            if (typeof data.type !== "undefined" && data.type !== null) {
                                if (data.type === $rootScope.rootData.pushConfig.pushType.NEW_MATCH) {
                                    $state.go("home.matches");
                                } else if (data.type === $rootScope.rootData.pushConfig.pushType.NEW_MESSAGE) {
                                    if (typeof data.userId !== "undefined" && data.userId !== null) {
                                        $state.go("home.messaging", {id: data.userId});
                                    } else {
                                        $state.go("home.matches");
                                    }
                                }
                            }
                            //console.log("Notification was received on device tray and tapped by the user");
                        } else {
                            //Notification was received in foreground. Maybe the user needs to be notified.
                            //console.log("Notification was received in foreground. Maybe the user needs to be notified.");
                        }
                    },
                    function (msg) {
                        //console.log('onNotification callback successfully registered: ' + msg);
                        FCMPlugin.getToken(
                                function (token) {
                                    //console.log("Device Token: " + token);
                                    $localStorage.pushToken = token;
                                },
                                function (err) {
                                    //console.log('error retrieving token: ' + err);
                                }
                        )
                    },
                    function (err) {
                        //console.log('Error registering onNotification callback: ' + err);
                    }
            );

            $rootScope.checkForUpdate();
        }

        if (typeof $localStorage.uid !== "undefined") {
            $rootScope.getUserMatches();
//        $rootScope.getUserChats();
            //console.log("root fonksiyondan cagrıldı");
        }
    });
    
    $rootScope.checkForUpdate = function () {
        
            $cordovaAppVersion.getVersionNumber().then(function (version) {
                $rootScope.rootData.version = version;

                ref.child("app_config").child("version").once('value', function (versionSnap) {
                    //console.log(versionSnap.val());

                    if (version < versionSnap.val()) {
                        $ionicPopup.alert({
                            okText: 'OK',
                            okType: 'button-stable',
                            title: 'Update Your App',
                            template: 'You should update your app to use it. \n Your app is running on ' + version + ' \n There is a new version: ' + versionSnap.val()
                        }).then(function () {
                            $rootScope.checkForUpdate();
                        });
                    }
                });
            });
    };

    $scope.interests = 'We compare your  Facebook friends  with those  of your matches to display  any  common connections'.split('  ');

    /*Edit Profile*/
    $ionicModal.fromTemplateUrl('templates/modals/profile_edit.html', {
        scope: $scope,
        animation: 'slide-in-up'
    }).then(function (modal) {
        $scope.editProfileModal = modal;
    });

    $scope.openEditProfileModal = function () {
        $scope.editProfileModal.show();
    };

    $scope.closeEditProfileModal = function () {
        $scope.editProfileModal.hide();
    };

    $rootScope.showLoading = function () {
        $ionicLoading.show({
            template: '<ion-spinner icon="lines" class="spinner-calm"></ion-spinner>',
            animation: 'fade-in',
            showBackdrop: true,
            showDelay: 0,
            hideOnStateChange: true,
            duration: 20000
        });
    };

    $rootScope.hideLoading = function () {
        $ionicLoading.hide();
    };

    $rootScope.closeListeners = function () {

        if (typeof $localStorage.uid !== "undefined" && $localStorage.uid !== null) {

            ref.child("user_chats").child($localStorage.uid).once('value', function (userChatsSnap) {
                userChatsSnap.forEach(function (userChat) {
                    ref.child("chat_messages").child(userChat.getKey()).off();
                });
            });

            ref.child("matches").child($localStorage.uid).once('value', function (userMatchSnap) {
                userMatchSnap.forEach(function (userMatch) {
                    ref.child("profile_photo").child(userMatch.getKey()).off();
                    ref.child("user_profile").child(userMatch.getKey()).off();
                });
            });

            ref.child("user_properties").child($localStorage.uid).off();
            ref.child("user_chats").child($localStorage.uid).off();
            ref.child("matches").child($localStorage.uid).off();
            ref.child("user_profile").child($localStorage.uid).off();
            ref.child("profile_photo").child($localStorage.uid).off();
        }

    };

    $rootScope.rootLogout = function () {

        $rootScope.closeListeners();

        var browser = ionic.Platform.is('browser');
        if (browser) {
            $rootScope.clearLocalStorage();
            $state.go("welcome");
        } else {
            facebookConnectPlugin.logout(function (success) {
                $rootScope.clearLocalStorage();
                $state.go("welcome");
            }, function (error) {
                //console.log("Facebook çıkısı gerçekleşmedi");
                $rootScope.clearLocalStorage();
                $state.go("welcome");
            });
        }

    };

    $rootScope.deleteAccount = function () {
        var browser = ionic.Platform.is('browser');
        if (browser) {
            $rootScope.removeUserFromDB(angular.copy($localStorage.uid));
            $rootScope.clearLocalStorage();
        } else {
            facebookConnectPlugin.logout(function (success) {
                $rootScope.removeUserFromDB(angular.copy($localStorage.uid));
                $rootScope.clearLocalStorage();
            }, function (error) {
                //console.log("Facebook çıkısı gerçekleşmedi");
                $rootScope.removeUserFromDB(angular.copy($localStorage.uid));
                $rootScope.clearLocalStorage();
            });
        }
    };

    $rootScope.removeUserFromDB = function (uid) {

        var storageRef = firebase.storage().ref();

        ref.child($rootScope.rootData.newUser.USER_PROFILE_PHOTO_KEY).child(uid)
                .once('value', function (photoSnap) {
                    var photos = angular.copy(photoSnap.val());
                    if (typeof photos !== "undefined" && photos !== null) {
                        for (var i = 0; i < photos.length; i++) {
                            if (photos[i] !== $rootScope.rootData.defaultProfilePhoto) {
                                storageRef.child(uid).child(i.toString()).delete();
                            }
                        }
                    }
                    ref.child($rootScope.rootData.newUser.USER_PROFILE_PHOTO_KEY).child(uid).remove();
                });
        ref.child($rootScope.rootData.newUser.USER_PROFILE_PHOTO_KEY).child(uid).remove();
        ref.child($rootScope.rootData.newUser.USER_LOCATOIN_KEY).child(uid).remove();
        ref.child($rootScope.rootData.newUser.USER_PROFILE_KEY).child(uid).remove();
        ref.child($rootScope.rootData.newUser.USER_PROPERTIES_KEY).child(uid).remove();
        ref.child($rootScope.rootData.newUser.USER_LIKE_KEY).child(uid).remove();
        ref.child($rootScope.rootData.newUser.USER_MATCHES).child(uid).remove();
        $scope.systemLog($rootScope.rootData.logType.INFO, uid + " kodlu kullanıcı hesabını sildi.");
        $state.go("welcome");

        ref.child($rootScope.rootData.newUser.USER_CHATS).child(uid).once('value', function (userChatSnap) {
            angular.forEach(userChatSnap.val(), function (value, chatId) {

                var chatMembersRef = ref.child($rootScope.rootData.newUser.CHAT_MEMBERS).child(chatId);
                chatMembersRef.once('value', function (chatMemberSnap) {

                    angular.forEach(chatMemberSnap.val(), function (value, chatMemberId) {
                        if (chatMemberId !== uid) {
                            ref.child($rootScope.rootData.newUser.USER_CHATS).child(chatMemberId).child(chatId).remove();                           
                            ref.child($rootScope.rootData.newUser.USER_CHATS).child(uid).child(chatId).remove();
                            ref.child($rootScope.rootData.newUser.CHAT_MEMBERS).child(chatId).remove();
                            ref.child($rootScope.rootData.newUser.CHAT_MESSAGES).child(chatId).remove();
                            ref.child($rootScope.rootData.newUser.CHATS).child(chatId).remove();
                        }
                    });
                });
            });
        });

    };

    $rootScope.clearLocalStorage = function () {
        delete $localStorage.uid;
        delete $localStorage.photo;
        delete $localStorage.name;
    };


    $scope.systemLog = function (type, logStr) {

        var log = angular.copy($rootScope.rootData.logTemplate);
        log.type = type;
        log.log = logStr;
        log.created = new Date().getTime();
        ref.child($rootScope.rootData.newUser.SYSTEM_LOGS).push(log);
    };

    $scope.upadtePositions = function (lat, lon) {

        var geoFire = new GeoFire(ref.child($rootScope.rootData.newUser.USER_LOCATOIN_KEY));
        geoFire.set($localStorage.uid, [lat, lon]).then(function () {

        }, function (error) {

        });
    };


    $scope.convertTimestampToDate = function (timestamp) {
        return new Date(timestamp).toLocaleDateString();
    };

    $rootScope.sendNewMessageNotification = function (receiverToken, senderId, senderName, message) {
        var title = senderName + ":";
        var body = message;
        var data = {
            type: $rootScope.rootData.pushConfig.pushType.NEW_MESSAGE,
            userId: senderId
        };
        $rootScope.sendNotification(receiverToken, title, body, data);
    };

    $rootScope.sendNewMatchNotification = function (receiverToken, matchedName) {
        var title = "Badigo";
        var body = "You matched with " + matchedName + ". Click to chat now!";
        var data = {
            type: $rootScope.rootData.pushConfig.pushType.NEW_MATCH
        };
        $rootScope.sendNotification(receiverToken, title, body, data);
    };

    $rootScope.sendNotification = function (receiverToken, title, body, data) {
        PushService.notification().send({
            "notification": {
                "title": title,
                "body": body,
                "sound": "default",
                "click_action": "FCM_PLUGIN_ACTIVITY",
                "color": "#000000"
            },
            "data": data,
            "to": receiverToken,
            "priority": "high",
            "restricted_package_name": $rootScope.rootData.packageName
        }, function (data) {
            //console.log(data);
        }, function (err) {
            //console.log(err);
        });
    };

    $scope.forceLocationErrorOpen = false;
    $scope.forceGelolocation = function () {
        $cordovaGeolocation
                .getCurrentPosition($rootScope.rootData.geolocationOptions)
                .then(function (position) {
                    $scope.forceLocationErrorOpen = false;
                    var lat = position.coords.latitude;
                    var lon = position.coords.longitude;
                    $scope.upadtePositions(lat, lon);

                }, function (err) {
                    //$scope.upadtePositions($rootScope.rootData.defaultCoords.lat, $rootScope.rootData.defaultCoords.lon);
                    $scope.forceLocationErrorOpen = true;
                    $ionicPopup.alert({
                        okText: 'OK',
                        okType: 'button-assertive',
                        title: 'Location Finding Failed',
                        template: 'If you want to use Badigo you must open your location info for it.'
                    }).then(function () {
                        $scope.forceGelolocation();
                    });
                });
    }

    $timeout(function () {
        if (typeof $localStorage.uid !== "undefined" && $localStorage.uid !== null) {
            $scope.forceGelolocation();
        }
    }, 2500);


    $ionicPlatform.on('resume', function (event) {
        if (typeof $localStorage.uid !== "undefined" && $localStorage.uid !== null && !$scope.forceLocationErrorOpen) {
            $scope.forceGelolocation();

            ref.child("user_status").child($localStorage.uid).set(true);
        }
    });


    $ionicPlatform.on('pause', function (event) {
        if (typeof $localStorage.uid !== "undefined" && $localStorage.uid !== null) {

            ref.child("user_status").child($localStorage.uid).set(false);
        }
    });


    var connectedRef = firebase.database().ref('.info/connected');
    connectedRef.on('value', function (snap) {

        //console.log("connected cagırıldı");
        if (snap.val() === true) {
            //console.log("connected cagırıldı true");

            if (typeof $localStorage.uid !== "undefined" && $localStorage.uid !== null) {

                ref.child("user_status").child($localStorage.uid).set(true);
                ref.child("user_status").child($localStorage.uid).onDisconnect().set(false);
            }

        }
    });


    $rootScope.getUserMatches = function () {

        $rootScope.rootData.totalNewMatch = 0;
        $rootScope.rootData.matches = [];


        if (window.cordova && window.cordova.plugins.notification.badge) {
            $cordovaBadge.clear().then(function () {
                // You have permission, badge cleared.
            }, function (err) {
                // You do not have permission.
            });
        }


        var matchRef = ref.child($rootScope.rootData.newUser.USER_MATCHES).child($localStorage.uid);
        matchRef.off();

        matchRef.on('child_added', function (match) {
            //console.log("---- getUserMatches child_added logStart ----");
            //console.log(match.val());
            //console.log(match.getKey());
            //console.log("---- getUserMatches child_added logEnd ----");

            var isNew = match.val().isNew;
            var matchTime = match.val().timestamp;
            var matchId = match.getKey();

            var matchedProfileRef = ref.child("profile_photo").child(matchId);
            matchedProfileRef.off();

            var matchedProfile = $firebaseArray(matchedProfileRef);
            matchedProfile.$loaded().then(function () {

                var profileNameRef = ref.child("user_profile").child(matchId).child("first_name");

                var profileName = $firebaseObject(profileNameRef);
                profileName.$loaded().then(function () {

                    $rootScope.rootData.matches.push({
                        name: profileName.$value,
                        isNew: isNew,
                        image: matchedProfile,
                        matchId: matchId,
                        matchTime: matchTime
                    });

                    if (isNew) {
                        $rootScope.rootData.totalNewMatch = $rootScope.rootData.totalNewMatch + 1;

                        if (window.cordova) {
                            $cordovaBadge.increase(1).then(function () {
                                // You have permission, badge increased.
                            }, function (err) {
                                // You do not have permission.
                            });
                        }

                    }
                });
            });

        });

        matchRef.on('child_changed', function (data) {
            //console.log("---- getUserMatches child_changed logStart ----");
            //console.log(data.val());
            //console.log(data.getKey());
            //console.log("---- getUserMatches child_changed logEnd ----");

            $rootScope.rootData.matches.some(function (value, index) {
                //console.log(value);

                if (value.matchId === data.getKey()) {
                    value.isNew = data.val().isNew;
                    return true;
                }
            });

        });

        matchRef.on('child_removed', function (data) {
            //console.log("---- getUserMatches child_removed logStart ----");
            //console.log(data.val());
            //console.log(data.getKey());
            //console.log("---- getUserMatches child_removed logEnd ----");

            var indexToRemove = -1;
            var removedIsNew = false;
            var userId = -1;

            $rootScope.rootData.matches.some(function (value, index) {
                //console.log(value);

                if (value.matchId === data.getKey()) {
                    indexToRemove = index;
                    removedIsNew = value.isNew;
                    userId = value.matchId;
                    return true;
                }
            });

            if (indexToRemove > -1) {
                $rootScope.rootData.matches.splice(indexToRemove, 1);

                if (removedIsNew) {
                    $rootScope.rootData.totalNewMatch = $rootScope.rootData.totalNewMatch - 1;

                    if (window.cordova) {
                        $cordovaBadge.decrease(1).then(function () {
                            // You have permission, badge increased.
                        }, function (err) {
                            // You do not have permission.
                        });
                    }

                }

                $rootScope.$broadcast('matchRemoved', {matchId: userId});

            }

        });

        $timeout(function () {
            $rootScope.getUserChats();
            $rootScope.$broadcast('matchFinished', {test: {}});
        }, 5000);

    };


    $rootScope.getUserChats = function () {

        var chatRef = ref.child($rootScope.rootData.newUser.USER_CHATS).child($localStorage.uid);
        chatRef.off();

        $rootScope.rootData.totalNewMessage = 0;
        $rootScope.rootData.chats = [];

        chatRef.on('child_added', function (data) {
            //console.log("---- getUserChats child_added logStart ----");
            //console.log(data.val());
            //console.log(data.getKey());
            //console.log("---- getUserChats child_added logEnd ----");

            var chatId = data.getKey();

            var chatMembersRef = ref.child($rootScope.rootData.newUser.CHAT_MEMBERS).child(chatId);

            chatMembersRef.once('value', function (chatMemberSnap) {

                angular.forEach(chatMemberSnap.val(), function (value, chatMemberId) {

                    if (chatMemberId !== $localStorage.uid) {

                        if ($rootScope.rootData.matches.length > 0) {

                            $rootScope.rootData.matches.some(function (value, index) {

                                if (value.matchId === chatMemberId) {

                                    var chatMsgRef = ref.child($rootScope.rootData.newUser.CHAT_MESSAGES).child(chatId);

                                    var sonMesaj = $firebaseArray(chatMsgRef.orderByChild("timestamp").limitToLast(1));

                                    sonMesaj.$loaded().then(function () {

                                        var newCount = 0;

                                        if (!sonMesaj[0].read && sonMesaj[0].userId !== $localStorage.uid) {
                                            newCount = newCount + 1;
                                            $rootScope.rootData.totalNewMessage = $rootScope.rootData.totalNewMessage + 1;
                                        }

                                        sonMesaj.$watch(function (event) {
                                            //console.log(event);
                                            if (event.event === 'child_added') {
                                                if (!sonMesaj[0].read && sonMesaj[0].userId !== $localStorage.uid) {
                                                    newCount = newCount + 1;
                                                    $rootScope.rootData.totalNewMessage = $rootScope.rootData.totalNewMessage + 1;
                                                }
                                            }

                                            if (event.event === 'child_changed') {
                                                if (sonMesaj[0].read && sonMesaj[0].userId !== $localStorage.uid) {
                                                    $rootScope.rootData.totalNewMessage = $rootScope.rootData.totalNewMessage - newCount;
                                                    newCount = 0;
                                                }
                                            }

                                        });

                                        $rootScope.rootData.chats.push({
                                            name: value.name,
                                            image: value.image,
                                            memberId: value.matchId,
                                            chatId: chatId,
                                            lastMessage: sonMesaj,
                                        });


                                    });

                                    return true;
                                }
                            });
                        }

                    }
                });

            });
        });

        chatRef.on('child_changed', function (data) {
            //console.log("---- getUserChats child_changed logStart ----");
            //console.log(data.val());
            //console.log(data.getKey());
            //console.log("---- getUserChats child_changed logEnd ----");
        });

        chatRef.on('child_removed', function (data) {
            //console.log("---- getUserChats child_removed logStart ----");
            //console.log(data.val());
            //console.log(data.getKey());
            //console.log("---- getUserChats child_removed logEnd ----");

            var indexToRemove = -1;

            $rootScope.rootData.chats.some(function (value, index) {

                if (value.chatId === data.getKey()) {
                    indexToRemove = index;
                    return true;
                }
            });

            if (indexToRemove > -1) {
                $rootScope.rootData.chats.splice(indexToRemove, 1);
            }
        });


        /*chatRef.on('value', function (user_chats) {
         
         $rootScope.rootData.chats = [];
         
         user_chats.forEach(function (user_chat) {
         
         var chatId = user_chat.getKey();
         
         var chatMembersRef = ref.child($rootScope.rootData.newUser.CHAT_MEMBERS).child(chatId);
         
         chatMembersRef.once('value', function (chat) {
         
         chat.forEach(function (member) {
         if (member.getKey() !== $localStorage.uid) {
         
         var memberId = member.getKey();
         
         var memberProfileRef = ref.child("profile_photo").child(memberId);
         memberProfileRef.off();
         
         var memberProfile = $firebaseArray(memberProfileRef);
         memberProfile.$loaded().then(function () {
         
         var firstNameRef = ref.child("user_profile").child(memberId).child("first_name");
         
         var profileName = $firebaseObject(firstNameRef);
         profileName.$loaded().then(function () {
         
         var chatMsgRef = ref.child($rootScope.rootData.newUser.CHAT_MESSAGES).child(chatId);
         
         var sonMesaj = $firebaseArray(chatMsgRef.orderByChild("timestamp").limitToLast(1));
         
         sonMesaj.$loaded().then(function () {
         
         var newCount = 0;
         
         if (!sonMesaj[0].read && sonMesaj[0].userId !== $localStorage.uid) {
         newCount = newCount + 1;
         $rootScope.rootData.totalNewMessage = $rootScope.rootData.totalNewMessage + 1;
         }
         
         sonMesaj.$watch(function (event) {
         //console.log(event);
         if (event.event === 'child_added') {
         if (!sonMesaj[0].read && sonMesaj[0].userId !== $localStorage.uid) {
         newCount = newCount + 1;
         $rootScope.rootData.totalNewMessage = $rootScope.rootData.totalNewMessage + 1;
         }
         }
         
         if (event.event === 'child_changed') {
         if (sonMesaj[0].read && sonMesaj[0].userId !== $localStorage.uid) {
         $rootScope.rootData.totalNewMessage = $rootScope.rootData.totalNewMessage - newCount;
         newCount = 0;
         }
         }
         
         });
         
         $rootScope.rootData.chats.push({
         name: profileName.$value,
         image: memberProfile[0].$value,
         memberId: memberId,
         lastMessage: sonMesaj
         });
         
         });
         });
         });
         }
         });
         });
         });
         });*/
    };

    $rootScope.findAdressFromGoogleResult = function (adressArray) {
        try {
            var codes = ['country', 'administrative_area_level_1', 'administrative_area_level_2'];
            var location = [];
            for (var i = 0; i < codes.length; i++) {
                for (var j = 0; j < adressArray.length; j++) {
                    var code = codes[i];
                    if (adressArray[j].types.indexOf(code) > -1) {
                        location.push(adressArray[j].short_name);
                        break;
                    }
                }
            }

            return location.join(", ");
        } catch (err) {
            $scope.systemLog($rootScope.rootData.logType.ERROR, $localStorage.uid + " kodlu kullanıcının konumu alınamadı. konum array: " + adressArray.toString());
            return "";
        }
    };

    $rootScope.goPage = function (pageId, direction) {
        $ionicViewSwitcher.nextDirection(direction);
        $state.go(pageId);
    };

}).filter('capitalize', function () {
    return function (input) {
        return (!!input) ? input.charAt(0).toUpperCase() + input.substr(1).toLowerCase() : '';
    };
}).filter('reverse', function () {
    return function (items) {
        return items.slice().reverse();
    };
}).filter('calendarFormat', function () {

    return function (time) {
        if (!time)
            return;

        return moment(time).calendar(null, {
            lastDay: '[Yesterday]',
            sameDay: 'LT',
            lastWeek: 'dddd',
            sameElse: 'DD/MM/YY'
        });
    };
}).directive('square', function () {
    return {
        restrict: 'A',
        link: function (scope, elem, attrs) {
            elem[0].height = elem[0].width;
        }
    };
}).directive('squareImage', function () {
    return {
        restrict: 'A',
        scope: {
            'imageUrl': '@'
        },
        link: function (scope, elem, attrs) {
            elem[0].height = elem[0].clientWidth;
            elem.css({
                'height': elem[0].clientWidth + 'px',
                'background-size': 'cover',
                'background-repeat': 'no-repeat',
                'background-position': 'top center'
            });

            scope.$watch('imageUrl', function (value) {
                if (typeof value !== "undefined" && value !== "") {
                    elem.css({
                        'background-image': 'url(' + value + ')',
                    });
                }
            });
        }
    };
}).directive('loadMoreAtTop', function ($timeout) {
    return {
        restrict: 'E',
        scope: {onScroll: '&onScroll'},
        require: '^$ionicScroll',
        template: function () {
            return '<ion-refresher pulling-text="More..." on-refresh="growBounds()"></ion-refresher>';
        },
        link: function ($scope, $element, attrs, scrollCtrl) {
            var scroller = scrollCtrl.$element[0];
            var lastHeight,
                    loading = false;
            $scope.growBounds = function growBounds() {
                loading = true;
                lastHeight = scroller.scrollHeight;
                //console.log('lastHeight', lastHeight);
                $scope.onScroll();
            }

            function resetScrollPosition() {
                var height = scroller.scrollHeight;
                var difference = height - lastHeight;
                //console.log('difference', difference, 'height', height, 'lastHeight', lastHeight);
                // reset the scroll based on previous height
                scrollCtrl.scrollTo(scrollCtrl.getScrollPosition().left, difference, true);
            }

            function finishGrowBounds() {
                if (loading) {
                    $timeout(function () {
                        loading = false;
                        resetScrollPosition();
                        $scope.$broadcast('scroll.refreshComplete');
                    }, 0);
                }
            }

            $scope.$parent.$on('scroll.loaded', finishGrowBounds);
        }
    };
});

/*.directive('squareImage', function ($sce) {
 return {
 restrict: 'A',
 replace: true,
 template: '<img />',
 scope: {
 'ngSrc': '@'
 },
 link: function (scope, elem, attrs) {
 var sqWidth = elem[0].clientWidth;
 elem[0].width = sqWidth;
 elem[0].height = sqWidth;
 var img = new Image();
 
 img.onload = function () {
 var canvas = document.createElement('canvas');
 if (img.width === img.height) {
 img.width = sqWidth;
 img.height = sqWidth;
 } else if (img.width > img.height) {
 var rat = sqWidth / img.height;
 img.width = img.width * rat;
 img.height = sqWidth;
 } else {
 var rat = sqWidth / img.width;
 img.width = sqWidth;
 img.height = img.height * rat;
 }
 canvas.width = sqWidth;
 canvas.height = sqWidth;
 var ctx = canvas.getContext('2d');
 ctx.drawImage(img, 0, 0, img.width, img.height);
 elem[0].crossOrigin = 'Anonymous';
 elem[0].src = canvas.toDataURL('image/jpeg', 1)
 scope.$apply();
 };
 
 scope.$watch('ngSrc', function (value) {
 img.crossOrigin = 'Anonymous';
 img.src = value;
 //console.log(value);
 });
 }
 };
 });*/


/*debug test için dursun
 *   (function(on) {
 firebase.database.Reference.prototype.on = function() {
 //console.log('.on()', arguments[0], this.path.toString());
 return on.apply(this, arguments);
 };
 })(firebase.database.Reference.prototype.on);
 
 (function(off) {
 firebase.database.Reference.prototype.off = function() {
 //console.log('.off()', arguments[0], this.path.toString());
 return off.apply(this, arguments);
 };
 })(firebase.database.Reference.prototype.off);
 
 firebase.database.enableLogging(function(msg) {
 if (msg.search(/listen/i) !== -1) // also catches "unlisten"
 //console.log('FB:',msg);
 });
 */