angular.module('BadigoApp').controller('ExploreControler', function ($rootScope, $scope, $ionicModal, firebase, $localStorage, $timeout, $state) {

    if (typeof $rootScope.onKeyEnteredRegistration !== "undefined" && $rootScope.onKeyEnteredRegistration !== null) {
        $rootScope.onKeyEnteredRegistration.cancel();
    }

    $scope.data = {
        user_profile: null,
        user_properties: null,
        photo: null,
        location: null,
        cards: [],
        matched: null,
        profileModal: {photos: []}
    };

    //$scope.data.profileModal.photos = ['images/noimage.png'];

    $scope.data.photo = $localStorage.photo;

    var ref = firebase.database().ref();
    var geoFire = new GeoFire(ref.child($rootScope.rootData.newUser.USER_LOCATOIN_KEY));

    $scope.exploreUser = function () {
        ref.child($rootScope.rootData.newUser.USER_PROFILE_KEY).child($localStorage.uid)
                .once('value', function (prof) {
                    $scope.data.user_profile = prof.val();
                    ref.child($rootScope.rootData.newUser.USER_PROPERTIES_KEY).child($localStorage.uid)
                            .once('value', function (props) {
                                $scope.data.user_properties = props.val();
                                $scope.startGeofire();
                            });
                });
    };

    $scope.startGeofire = function () {
        geoFire.get($localStorage.uid).then(function (location) {
            if (location === null) {
                //console.log("Provided key is not in GeoFire");
            } else {
                $scope.data.location = location;
                //console.log($scope.data);
                $scope.geoQuery = geoFire.query({
                    center: [$scope.data.location[0], $scope.data.location[1]],
                    radius: $scope.data.user_properties.max_distance
                });
                $rootScope.onKeyEnteredRegistration = $scope.geoQuery.on("key_entered", function (key, location, distance) {
                    //console.log(key + " entered query at " + location + " (" + distance + " km from center)");
                    if ($scope.data.user_profile.gender === $rootScope.rootData.gender.MALE) {
                        $scope.getFemaleUser(key, distance);
                    } else if ($scope.data.user_profile.gender === $rootScope.rootData.gender.FEMALE) {
                        $scope.getMaleUser(key, distance);
                    }

                });

            }
        }, function (error) {
            //console.log("Error: " + error);
        });
    };

    $scope.getMaleUser = function (userId, distance) {
        $scope.getUser(userId, distance, $rootScope.rootData.gender.MALE);
    };

// SADECE KENDISINI BEGENENLERI GETIREN FUNCTION
//    $scope.getFemaleUser = function (userId, distance) {
//        ref.child($rootScope.rootData.newUser.USER_LIKE_KEY).child(userId).child($localStorage.uid)
//                .once('value', function (foundProfileSnap) {
//                    if (foundProfileSnap.val()) {
//                        $scope.getUser(userId, distance, $rootScope.rootData.gender.FEMALE);
//                    }
//                });
//
//    };

    $scope.getFemaleUser = function (userId, distance) {
        $scope.getUser(userId, distance, $rootScope.rootData.gender.FEMALE);
    };


    $scope.getUser = function (userId, distance, searchGender) {
        if ($localStorage.uid === userId) {
            return;
        }

        ref.child($rootScope.rootData.newUser.USER_LIKE_KEY).child($localStorage.uid).child(userId)
                .once('value', function (profileSnap) {
                    if (profileSnap.val() === null) {
                        //console.log(profileSnap.val());
                        var now = new Date();

                        ref.child($rootScope.rootData.newUser.USER_PROFILE_KEY).child(userId)
                                .once('value', function (likedProfile) {
                                    var liked = likedProfile.val();
                                    var likedAge = now.getYear() - new Date(liked.birth).getYear();
                                    if (likedAge >= $scope.data.user_properties.min_age && likedAge <= $scope.data.user_properties.max_age &&
                                            liked.gender === searchGender) {
                                        //console.log(liked.first_name + " kriterlere uygun " + new Date(liked.birth) + "distance: " + distance.toFixed(2) + " km from center");
                                        ref.child($rootScope.rootData.newUser.USER_PROFILE_PHOTO_KEY).child(userId)
                                                .once('value', function (photoSnap) {
                                                    var photos = photoSnap.val();
                                                    ref.child($rootScope.rootData.newUser.USER_PROPERTIES_KEY).child(userId).child("location")
                                                            .once('value', function (locationSnap) {
                                                                var arr = [];
                                                                for (var i = 0; i < photos.length; i++) {
                                                                    if (photos[i] !== $rootScope.rootData.defaultProfilePhoto) {
                                                                        arr.push(photos[i]);
                                                                    }
                                                                }

                                                                $scope.$apply(function () {
                                                                    if ($scope.data.cards.length < $rootScope.rootData.exploreMaxUser) {
                                                                        $scope.data.cards.push({
                                                                            key: userId,
                                                                            photos: arr,
                                                                            profile: liked,
                                                                            age: likedAge,
                                                                            distance: distance.toFixed(2),
                                                                            location: locationSnap.val(),
                                                                            likeIntersection: $scope.likeIntersection(liked.likes, $scope.data.user_profile.likes)
                                                                        });
                                                                    }
                                                                    $scope.cardsControl();
                                                                });
                                                                //console.log($scope.data.cards.length);
                                                            });
                                                });

                                    } else {
                                        //console.log(liked.first_name + "-" + liked.gender + " kriterlere uygun değil " + new Date(liked.birth) + "distance: " + distance.toFixed(2) + " km from center");
                                    }
                                });
                    }
                });
    };



    /*$cordovaGeolocation
     .getCurrentPosition($rootScope.rootData.geolocationOptions)
     .then(function (position) {
     geoFire.set($localStorage.uid, [position.coords.latitude, position.coords.longitude]).then(function () {
     //console.log("Provided key has been added to GeoFire");
     }, function (error) {
     //console.log("Error: " + error);
     });
     }, function (err) {
     //console.log("Error: " + err);
     });*/

    $scope.likeOrdered = function () {
        var card = $scope.data.cards[0];
        $scope.transitionRight(card);
        $scope.cardDestroyed(0);
    };

    $scope.unlikeOrdered = function () {
        var card = $scope.data.cards[0];
        $scope.transitionLeft(card);
        $scope.cardDestroyed(0);
    };

    $scope.cardDestroyed = function (index) {
        $scope.data.cards.splice(index, 1);
        $scope.isMoveLeft = false;
        $scope.isMoveRight = false;
        $scope.cardsControl();
        //console.log("Total Card Num: " + $scope.data.cards.length);
    };

    $scope.cardPartialSwipe = function (amt) {
        $scope.isMoveLeft = amt < -0.15;
        $scope.isMoveRight = amt > 0.15;
    }

    $scope.transitionRight = function (card) {
        ref.child($rootScope.rootData.newUser.USER_LIKE_KEY).child($localStorage.uid).child(card.key).set(true, function (error) {
            if (error) {

            } else {

                ref.child($rootScope.rootData.newUser.USER_LIKE_KEY).child(card.key).child($localStorage.uid).once('value', function (isMatchSnap) {
                    if (isMatchSnap !== null && isMatchSnap.val()) {
                        ref.child($rootScope.rootData.newUser.USER_PROPERTIES_KEY).child(card.key).child("new_match")
                                .once('value', function (matchNotificationSnap) {
                                    ref.child("matches/").child($localStorage.uid).child(card.key).set({isNew: true, timestamp: firebase.database.ServerValue.TIMESTAMP});
                                    ref.child("matches/").child(card.key).child($localStorage.uid).set({isNew: true, timestamp: firebase.database.ServerValue.TIMESTAMP});

                                    $scope.data.matched = card;
                                    if (matchNotificationSnap !== null && matchNotificationSnap.val()) {
                                        $rootScope.sendNewMatchNotification(card.profile.push_token, $scope.data.user_profile.first_name);
                                    }

                                });
                                
                        $scope.openMatchModal();
                    }
                });

//// SADECE KENDISINI BEGENENLERI GETIREN FUNCTION
//                if ($scope.data.user_profile.gender === $rootScope.rootData.gender.MALE) {
//                    ref.child($rootScope.rootData.newUser.USER_PROPERTIES_KEY).child(card.key).child("new_match")
//                            .once('value', function (matchNotificationSnap) {
//                                ref.child("matches/").child($localStorage.uid).child(card.key).set({isNew: true, timestamp: firebase.database.ServerValue.TIMESTAMP});
//                                ref.child("matches/").child(card.key).child($localStorage.uid).set({isNew: true, timestamp: firebase.database.ServerValue.TIMESTAMP});
//
//                                $scope.data.matched = card;
//                                if (matchNotificationSnap !== null && matchNotificationSnap.val()) {
//                                    $rootScope.sendNewMatchNotification(card.profile.push_token, $scope.data.user_profile.first_name);
//                                }
//
//                            });
//                }
            }
        });
        

// SADECE KENDISINI BEGENENLERI GETIREN FUNCTION
//        if ($scope.data.user_profile.gender === $rootScope.rootData.gender.MALE) {
//            $scope.openMatchModal();
//        }

        //console.log('card removed to the right');
        //console.log(card);

    };

    $scope.transitionLeft = function (card) {
        ref.child($rootScope.rootData.newUser.USER_LIKE_KEY).child($localStorage.uid).child(card.key).set(false, function (error) {
            if (error) {

            } else {

            }
        });
        //console.log('card removed to the left');
        //console.log(card);

    };

    $scope.cardsControl = function () {
        if ($scope.data.cards.length >= $rootScope.rootData.exploreMaxUser) {
            $rootScope.onKeyEnteredRegistration.cancel();
        } else if ($scope.data.cards.length <= 0) {
            $scope.startGeofire();
        }
    };

    // Match popup
    $ionicModal.fromTemplateUrl('templates/modals/match.html', {
        scope: $scope,
        animation: 'slide-in-up'
    }).then(function (modal) {
        $scope.matchModal = modal;
    });

    $scope.openMatchModal = function () {
        $scope.matchModal.show();
    };

    $scope.openChatMessages = function (userId) {

        //$state.go('home.messaging({id: userId})');
        $state.go('home.messaging', {'id': userId});
        $scope.closeMatchModal();
    };

    $scope.closeMatchModal = function () {
        $scope.data.matched = null;
        $scope.matchModal.hide();
    };

    $ionicModal.fromTemplateUrl('templates/modals/profile.html', {
        scope: $scope,
        animation: 'slide-in-up'
    }).then(function (modal) {
        $scope.profileModal = modal;
    });

    $scope.openProfileModal = function (card) {

        $scope.data.profileModal = {
            key: card.userId,
            photos: card.photos,
            first_name: card.profile.first_name,
            last_name: card.profile.last_name,
            age: card.age,
            aboutMe: card.profile.about,
            distance: card.distance,
            likesIntersection: card.likeIntersection
        };

        $scope.profileModal.show();

    };

    $scope.closeProfileModal = function () {
        $scope.profileModal.hide();
    };

    $scope.likeIntersection = function (a, b) {
        var arr = [];

        if (a !== undefined && b !== undefined) {
            for (var i = 0; i < a.length; i++) {
                for (var j = 0; j < b.length; j++) {
                    if (a[i].id === b[j].id) {
                        arr.push(a[i]);
                    }
                }
            }
        }

        return arr;
    };



    $scope.$on("$ionicView.beforeEnter", function (event, data) {
        $timeout(function () {
            $scope.exploreUser();
        }, $rootScope.rootData.exploreTimeout);
    });

});