angular.module('BadigoApp').controller('UserSettingsController', function ($rootScope, $scope, firebase, $localStorage, $firebaseObject, $ionicModal,
        $ionicPopup, PushService, $cordovaGeolocation, $timeout, $cordovaInAppBrowser) {

    $rootScope.showLoading();
    $scope.data = {
        slider: {
            options: {
                floor: $rootScope.rootData.age.min,
                ceil: $rootScope.rootData.age.max,
                minRange: 5,
                pushRange: true,
                translate: function (value) {
                    return '';
                }
            }
        },
        distanceSlider: {
            options: {
                floor: $rootScope.rootData.distance.min,
                ceil: $rootScope.rootData.distance.max,
                translate: function (value) {
                    return '';
                }
            }
        },
        genders: [
            {
                name: "Man",
                key: $rootScope.rootData.gender.MALE,
                value: true
            }, {
                name: "Woman",
                key: $rootScope.rootData.gender.FEMALE,
                value: true
            }
        ],
        genderText: "",
        user_properties: {},
        location: null
    };
    var ref = firebase.database().ref();
    var geoFire = new GeoFire(ref.child($rootScope.rootData.newUser.USER_LOCATOIN_KEY));
    $scope.startGeofire = function () {
        geoFire.get($localStorage.uid).then(function (location) {
            $scope.data.location = location;
        }, function (error) {
            //console.log("Error: " + error);
        });
    };
    $scope.checkLookingFor = function () {
        for (var i = 0; i < $scope.data.genders.length; i++) {
            if ($scope.data.user_properties.gender.indexOf($scope.data.genders[i].key) > -1) {
                $scope.data.genders[i].value = true;
            } else {
                $scope.data.genders[i].value = false;
            }
        }
        $scope.checkLookingForGenderText();
    };
    $scope.checkLookingForValue = function () {
        $scope.data.user_properties.gender = [];
        for (var i = 0; i < $scope.data.genders.length; i++) {
            if ($scope.data.genders[i].value) {
                $scope.data.user_properties.gender.push($scope.data.genders[i].key);
            }
        }
        $scope.checkLookingForGenderText();
    };
    $scope.checkLookingForGenderText = function () {
        var str = "";
        if ($scope.data.user_properties.gender.length === 1) {
            str = "Only ";
        }
        var nameArr = [];
        for (var i = 0; i < $scope.data.genders.length; i++) {
            if ($scope.data.genders[i].value) {
                nameArr.push($scope.data.genders[i].name);
            }
        }
        str += nameArr.join(", ");
        $scope.data.genderText = str;
    };
    $scope.data.user_properties = $firebaseObject(ref.child($rootScope.rootData.newUser.USER_PROPERTIES_KEY).child($localStorage.uid));
    $scope.data.user_properties.$loaded().then(function () {
        $rootScope.hideLoading();
        $scope.checkLookingFor();
    });
    $scope.logout = function () {
        $ionicPopup.confirm({
            title: 'Log Out',
            template: 'Are you sure you want to logout?',
            cancelText: 'Cancel',
            cancelType: 'button-dark',
            okText: 'OK',
            okType: 'button-stable'
        }).then(function (res) {
            if (res) {
                $rootScope.rootLogout();
            }
        });
    };
    $scope.deleteAccount = function () {
        $ionicPopup.confirm({
            title: 'Delte Account',
            template: 'Are you sure you want to delete this account?',
            cancelText: 'Cancel',
            cancelType: 'button-dark',
            okText: 'OK',
            okType: 'button-stable'
        }).then(function (res) {
            if (res) {
                $rootScope.deleteAccount();
            }
        });
    };
    $scope.getLocationName = function () {
        $rootScope.showLoading();
        $timeout(function () {
            $cordovaGeolocation
                    .getCurrentPosition($rootScope.rootData.geolocationOptions)
                    .then(function (position) {
                        var lat = position.coords.latitude;
                        var lon = position.coords.longitude;
                        PushService.geocode().translate({
                            latlng: lat + "," + lon
                        }, function (data) {
                            $rootScope.hideLoading();
                            var adress = data.results[0].address_components;
                            var adressStr = $rootScope.findAdressFromGoogleResult(adress);
                            $scope.data.user_properties.location = adressStr;
                            //$scope.data.user_properties.location = adress[5].short_name + ", " + adress[4].short_name + ", " + adress[3].short_name;
                        }, function (err) {
                            $rootScope.hideLoading();
                        });
                        geoFire.set($localStorage.uid, [lat, lon]).then(function () {

                        }, function (error) {

                        });
                    }, function (err) {
                        $rootScope.hideLoading();
                        $ionicPopup.alert({
                            okText: 'OK',
                            okType: 'button-assertive',
                            title: 'Location Finding Failed',
                            template: 'Error while finding your location, please try again later.'
                        });
                    });
        }, 200);
    };


    $scope.helpAndSupport = function () {
        $cordovaInAppBrowser.open($rootScope.rootData.staticContent.rootUrl, '_system', {
            location: 'yes',
            clearcache: 'yes',
            toolbar: 'no'
        }).then(function (event) {
            // success
        }).catch(function (event) {
            // error
        });
    };

    /************************Looking For Modal*************************/
    $ionicModal.fromTemplateUrl('view/usersettings/lookingfor.html', {
        scope: $scope,
        animation: 'slide-in-up'
    }).then(function (modal) {
        $scope.lookingForModal = modal;
    });
    $scope.openLookingForModal = function () {
        $scope.lookingForModal.show();
    };
    $scope.closeLookingForModal = function () {
        $scope.lookingForModal.hide();
    };
    /************************Privacy Modal*************************/
    $ionicModal.fromTemplateUrl('view/usersettings/privacy.html', {
        scope: $scope,
        animation: 'slide-in-up'
    }).then(function (modal) {
        $scope.privacyModal = modal;
    });
    $scope.data.privacyContent = null;
    $scope.openPrivacyModal = function () {
        $rootScope.showLoading();
        if ($scope.data.privacyContent !== null) {
            $rootScope.hideLoading();
            $scope.privacyModal.show();
        } else {
            PushService.privacy().read({}, function (data) {
                $rootScope.hideLoading();
                $scope.data.privacyContent = data.html;
                $scope.privacyModal.show();
            }, function (error) {
                $rootScope.hideLoading();
                $ionicPopup.alert({
                    okText: 'OK',
                    okType: 'button-assertive',
                    title: 'Privacy Policy Failed',
                    template: 'Error while opening Privacy Policy page, please try again later.'
                });
            });
        }
    };
    $scope.closePrivacyModal = function () {
        $scope.privacyModal.hide();
    };
    /************************Terms Modal*************************/
    $ionicModal.fromTemplateUrl('view/usersettings/terms.html', {
        scope: $scope,
        animation: 'slide-in-up'
    }).then(function (modal) {
        $scope.termsModal = modal;
    });
    $scope.data.termsContent = null;
    $scope.openTermsModal = function () {
        $rootScope.showLoading();
        if ($scope.data.termsContent !== null) {
            $rootScope.hideLoading();
            $scope.termsModal.show();
        } else {
            PushService.terms().read({}, function (data) {
                $rootScope.hideLoading();
                $scope.data.termsContent = data.html;
                $scope.termsModal.show();
            }, function (error) {
                $rootScope.hideLoading();
                $ionicPopup.alert({
                    okText: 'OK',
                    okType: 'button-assertive',
                    title: 'Terms of Service Failed',
                    template: 'Error while opening Terms of Service page, please try again later.'
                });
            });
        }
    };
    $scope.closeTermsModal = function () {
        $scope.termsModal.hide();
    };
    $scope.$on('$destroy', function () {
        $scope.lookingForModal.remove();
        $scope.privacyModal.remove();
        $scope.termsModal.remove();
    });
    $scope.$on('$ionicView.beforeLeave', function () {
        $scope.data.user_properties.$save();
    });

    $scope.startGeofire();
});